//-----------------------------------------------------------------------
// ボール打ちゲーム（自由落下アニメ＋角打ち）
// D0に接続されたスイッチでラケットを左へ
// D1に接続されたスイッチでラケットを右へ
//-----------------------------------------------------------------------
// Copyright (C) 2015  Takao Satoh  All Right Reserved
//-----------------------------------------------------------------------

//ライブラリを取り込む
#include <Sprite.h>
#include <Matrix.h>
#include <Button.h>
#include <RPlib.h>
#include <Tone.h>

Matrix myMatrix = Matrix(11, 13, 1); //LEDマトリクスを使う前のおまじない
Button SW1 = Button(D0, HIGH); //D0に接続されたスイッチをSW1と命名する
Button SW2 = Button(D1, HIGH); //D1に接続されたスイッチをSW2と命名する
Tone tone1;

char rx;  //ラケットx座標
char x;  //ボールx座標
char y;  //ボールy座標
char dx;  //ボールx増分
char dy;  //ボールy増分

int i;  //ループカウンタ
int start = -1;  //初期状態
int speed, score, move;

void setup()
{
  myMatrix.setBrightness(1);  //マトリクスの明るさ（０−８）
  tone1.begin(D2);  //D2にスピーカを繋げる
}

void loop(){
  if (start < 0) {
    rx = 3;  x = 3;  y = 6;  dy = -1;
    myMatrix.clear();
//    myMatrix.write(x, y, HIGH);
    i = 0;  score = 0;  move = 0;
    racket();
    fall();
    start = 1;
  }
  SW1.listen();  //SW1の状態をマイコンに聞く
  SW2.listen();  //SW2の状態をマイコンに聞く

  if (SW1.onPress()) {  //SW1が押されたら
    move = -1;
    if (start) {
      dx = -1;
      start = 0;
    }
    racket();
  } else if (SW2.onPress()) {  //SW2が押されたら
    move = 1;
    if (start) {
      dx = 1;
      start = 0;
    }
    racket();
  }

  if (! start) {
    speed = (score < 30) ?  40 - score :  10;
    if (++i > speed) {
//      tone1.play(NOTE_F4, 10);  //ファの音を10msec出す
      ball();
      i = 0;
    }
  }
  delay(10);
}

void racket() {
  if (move < 0 && rx > 0) {
    myMatrix.write(rx+1, 7, LOW);
    myMatrix.write(--rx, 7, HIGH);
  } else if (move > 0 && rx < 6) {
    myMatrix.write(rx, 7, LOW);
    myMatrix.write(++rx+1, 7, HIGH);
  } else {  // move == 0
    myMatrix.write(rx, 7, HIGH);
    myMatrix.write(rx+1, 7, HIGH);
  }
}

void fall() {
  char h;  //計算したy座標
  int i, t, t0 = sqrt(600), h0 = 0;

  x = 3;  y = 0;
  myMatrix.write(x, y, HIGH);
  delay(500);

  for (t = t0;  ; t++) {
    i = t - t0;
    h = (h0 + i * i) / 100;
    if (i > 0 && h >= 6) {
      h = 6;
      t0 = i * 0.8;  //減衰（時間軸切り詰め）
      h0 = 600 - t0 * t0;  //y座標かさ上げ
      t = 0;
      tone1.play(NOTE_F4, 10);  //バウンド音
    }
    if (h != y) {
      myMatrix.write(x, y, LOW);
      myMatrix.write(x, h, HIGH);
      y = h;
    }
    if (t0 == 0) {
      break;
    }
    delay(12);
  }
}

void ball() {
  myMatrix.write(x, y, LOW);

  y += dy;
  if (y > 6) {
    if (y > 7) {
      move = 0;  racket();  //ラケット強制再表示
      tone1.play(NOTE_D4, 200);  delay(200);
      tone1.play(NOTE_D4, 400);
      myMatrix.clear();
      myMatrix.putd2(1, 1, score);  //２桁スコア数字を表示
      delay(5000);
      start = -1;
    } else if (x < rx || x > rx+1) {
      if ((x == rx-1 && dx > 0) || (x == rx+2 && dx < 0)) {  // 角打ち
        tone1.play(NOTE_A4, 10);  //打音
        dx *= -1;
        dy = -1;
        y = 5;
        score++;
      } else {  // MISSING
        tone1.play(NOTE_F4, 10);  //移動音
      }
    } else {
      tone1.play(NOTE_A4, 10);  //打音
      dy = -1;
      y = 5;
      score++;
    }
  } else {
    tone1.play(NOTE_F4, 10);  //移動音
    if (y < 0) {
      dy = 1;
      y = 1;
    }
  }

  x += dx;
  if (x > 7) {
    dx = -1;
    x = 6;
  } else if (x < 0) {
    dx = 1;
    x = 1;
  }

  if (! start) {
    myMatrix.write(x, y, HIGH);
  }
}
